/* Biblioteka odpowiedzialna za komunikację z wyswietlaczem dotykowym */

#include "Arduino.h"
#include <stdlib.h>
#include "Marlin.h"
#include "ultralcd.h"
#include "planner.h"
#include "stepper.h"
#include "temperature.h"
#include "motion_control.h"
#include "cardreader.h"
#include "watchdog.h"
#include "ConfigurationStore.h"
#include "language.h"
#include "pins_arduino.h"
#include "TFTDispComm.h"

// definicje parametrów do odczytu
#define DISP_ENABLE 1
#define TEMP_EXTR0 10
#define TEMP_EXTR1 11
#define TEMP_BED 12
#define CARD_NUMBEROFFILES 50
#define CARD_LIST 51
#define CARD_INIT 60
#define CARD_SETPRINTFILE 61
#define CARD_STARTPRINTING 62
#define CARD_PAUSEPRINTING 63
#define CARD_RESUMEPRINTING 64
#define CARD_PRINTINGPROGRESS 65
#define CARD_ISPRINTING 66

// deklaracje zmiennych do sprawdzenia dostępności wyświetlacza
unsigned long pronterfaceTime;
bool dispEnable;

// **************** DEKLARACJE FUNKCJI ****************

// deklaracje funkcji komunikacyjnych
void readDataFromDispTFT(void);
void sendStrToDispTFT(int parameter, char *str);
void sendIntToDispTFT(int parameter, int data);

// deklaracje funkcji do odczytu temperatury
void readTempBed(void);
void readTempExtr1(void);
void readTempExtr0(void);

// deklaracje dunkcji do obsługi karty SD
void cardInit(void);
void setCardPrintFile(int number);
void readCardList(void);
void readCardNumberOfFiles(void);
void cardStartPrinting(void);
void cardPausePrinting(void);
void readCardPrintingProgress(void);
void readCardIsPrinting(void);

// **************** FUNKCJE GŁÓWNE ****************

// Inicjalizacja komunikacji z wyswietlaczem dotykowym
void TFTDispCommInit(void)
{
  // inicjalizacja UART
  // Serial2 jest wykorzystywany do komunikacji z wyświetlaczem LCD
  Serial2.begin(9600);
  dispEnable = 1;
}

// Sprawdzanie, czy wyświetlacz nie przesłał polecenia
void TFTDispCommProcess(void)
{    
  while( Serial2.available() )  
  {
    readDataFromDispTFT();
  }
  
  // sprawdzanie, czy Disconnet z Pronterface'em
  if( millis() - pronterfaceTime > 3000 ) {
    dispEnable = 1;
  }
  
}

// **************** WYŚWIETLACZ ****************

// Odczyt, czy wyświetlacz jest dostępny
void readDispEnable(void) {
  sendIntToDispTFT( DISP_ENABLE, dispEnable );
}

// **************** TEMPERATURA ****************

// Odczyt temperatury Extrudera 0
void readTempExtr0(void) {
  sendIntToDispTFT( TEMP_EXTR0, degHotend(0) ); 
}

// Odczyt temperatury Extrudera 1
void readTempExtr1(void) {
  sendIntToDispTFT( TEMP_EXTR1, degHotend(1) );
}

// Odczyt temperatury Łoża
void readTempBed(void) {
  sendIntToDispTFT( TEMP_BED, degBed() ); 
}

// **************** KARTA SD ****************

// Inicjalizacja
void cardInit(void) {
  card.initsd();
}

// Odczyt ilości plików na karcie SD
void readCardNumberOfFiles(void) {
  sendIntToDispTFT( CARD_NUMBEROFFILES, card.getnrfilenames() - 1 );
}

// Odczyt nazw plików na karcie SD
void readCardList(void) {  
  int cardNumberOfFiles = card.getnrfilenames();
  Serial2.print('@');
  Serial2.print(CARD_LIST);
  Serial2.print(':'); 
  for(int i = 1 ; i < cardNumberOfFiles ; i++) {
    card.getfilename(i);
    Serial2.print( card.filename );
    Serial2.print('|');
  }
  Serial2.print('#'); 
}

// Wybranie pliku do druku
void setCardPrintFile(int number) {
  card.getfilename(number);
  card.openFile( card.filename, 1 ); 
}

// Rozpoczęcie drukowania
void cardStartPrinting(void) {
  card.startFileprint();
}

// Zatrzymanie drukowania
void cardPausePrinting(void) {
  card.pauseSDPrint();  
}

// Wznowienie drukowania
void cardResumePrinting(void) {
  card.startFileprint();
}

// Postęp wydruku
void readCardPrintingProgress(void) {
  uint64_t progress = ( ( card.getSdPos() * 100 ) /  card.getFileSize() );
  sendIntToDispTFT( CARD_PRINTINGPROGRESS, progress );
}

// Sprawdzanie, czy trwa wydruk z karty SD
void readCardIsPrinting(void) {
  if( card.getSdPrinting() ==  true )
    sendIntToDispTFT( CARD_ISPRINTING, 1 );
  else
    sendIntToDispTFT( CARD_ISPRINTING, 0 );
}

// **************** KOMUNIKACJA ****************

// Inicjalizacja UART
void uartInit(void) {
  	// obliczanie UBRR dla U2X=0
	uint16_t baud = F_CPU/16/UART2_BAUD-1;
	// ustawienie prędkości 
	UBRR1H = (uint8_t)(baud>>8); // 4 starsze bity
	UBRR1L = (uint8_t)baud; // 8 młodszych bitów
	// załączenie nadajnika i odbiornika
	UCSR1B = (1<<RXEN1) | (1<<TXEN1);
	// włączenie przerwań
	UCSR1B |= (1<<RXCIE1);
	// ustawienie formatu ramki: 8 bitów danych, 1 bit stopu
	UCSR1C = (1<<UCSZ11) | (1<<UCSZ10);

	sei(); // globalne odblokowanie przerwań
  
}
// Wysyłanie wartości liczbowej do wyświetlacza
void sendIntToDispTFT(int parameter, int data)
{
  Serial2.print('@');
  Serial2.print(parameter);
  Serial2.print(';');
  Serial2.print(data);
  Serial2.print('#');
}

// Wysyłanie tekstu do wyświetlacza
void sendStrToDispTFT(int parameter, char *str)
{
  Serial2.print('@');
  Serial2.print(parameter);
  Serial2.print(':');
  Serial2.print(str);
  Serial2.print('#');  
}

// Odbiór danych z wyświetlacza
// *28# G-kod
// %79# - polecenie
// $33# - polecenie zapisu
// @32# - polecenie odczytu
void readDataFromDispTFT(void)
{
  char c;
  static int readState;
  
  // odczytany parametr
  static char tabPar[10];
  static int cntPar;
  
  // odczytana wartość
  static char tabWart[33]; 
  static int cntWart;
  
  // odczytywanie początku: 
  while( Serial2.available() && readState == 0 )
  {
    c = Serial2.read();
    if( c == '*' ) // G-kod
    {
      readState = 10;
      cntPar = 0;
    }
    if( c == '%' ) // polecenie
    {
      readState = 20;
      cntPar = 0; 
    }
    
    if( c == '@' ) // polecenie odczytu
    {
      readState = 30;
      cntPar = 0;  
    }
    
    if( c == '$' ) // polecenie zapisu
    {
      readState = 40;
      cntPar = 0; 
      cntWart = 0; 
    }
  }
  
  /* Jeśli odebrano G-kod */
  
  // odczytanie G-kodu i znaku końca '#'
  while( Serial2.available() && readState == 10 )
  {
    c = Serial2.read();
    if( c != '#')
    {
      tabPar[cntPar] = c;
      cntPar++;
    }
    else
      readState = 11;
  }
  
  // odczytanie znaku "\n"
  if( Serial2.available() && readState == 11 )
  {
    c = Serial2.read();
    if( c == '\n' )
    {
      tabPar[cntPar] = c;
      cntPar++;
      tabPar[cntPar] = NULL;
      readState = 12;
    }
  }
  
  // wykonanie G-kodu
  if( readState == 12 )
  {
    readState = 0;
    enquecommand( tabPar );
  }
  
  /* Jeśli odebrano polecenie */
  
  // odczytanie parametru
  while( Serial2.available() && readState == 20 )
  {
    c = Serial2.read();
    if( c != '#')
    {
      tabPar[cntPar] = c;
      cntPar++;
    }
    else
    {
      tabPar[cntPar] = NULL;
      readState = 21;
    }
  }
  
  // interpretacja polecenia
  if( readState == 21 )
  {
    readState = 0;
    switch( atoi(tabPar) )
    {
      case CARD_INIT : cardInit();
        break;
      case CARD_STARTPRINTING : cardStartPrinting();
        break;
      case CARD_PAUSEPRINTING : cardPausePrinting();
        break;
      case CARD_RESUMEPRINTING : cardResumePrinting();
        break;
    }
  }
  
  /* Jeśli odebrano polecenie odczytu */
  
  // odczytanie parametru
  while( Serial2.available() && readState == 30 )
  {
    c = Serial2.read();
    if( c != '#')
    {
      tabPar[cntPar] = c;
      cntPar++;
    }
    else
    {
      tabPar[cntPar] = NULL;
      readState = 31;
    }
  }  
 
  // interpretacja polecenia odczytu
  if( readState == 31 )
  {
    readState = 0;
    switch( atoi(tabPar) )
    {
      case TEMP_EXTR0 : readTempExtr0();
        break;
      case TEMP_EXTR1 : readTempExtr1();
        break;
      case TEMP_BED : readTempBed();
        break;
      case CARD_NUMBEROFFILES : readCardNumberOfFiles();  
        break;
      case CARD_LIST : readCardList();
        break;  
      case CARD_PRINTINGPROGRESS : readCardPrintingProgress();
        break;
      case CARD_ISPRINTING : readCardIsPrinting();
        break;
      case DISP_ENABLE : readDispEnable();
        break;
    }
  } 
  
  /* Jeśli odebrano polecenie zapisu */
  
    // odczytanie parametru
  while( Serial2.available() && readState == 40 )
  {
    c = Serial2.read();
    if( c != ';')
    {
      tabPar[cntPar] = c;
      cntPar++;
    }
    else
    {
      tabPar[cntPar] = NULL;
      readState = 41;
    }
  } 
  
  // odczytanie wartości
  while( Serial2.available() && readState == 41 )
  {
    c = Serial2.read();
     if( c != '#' )
    {
      tabWart[cntWart] = c;
      cntWart++;
    }
    else
    {
      tabWart[cntWart] = NULL;
      readState = 42; 
    }
  }
  
  // interpretacja polecenia zapisu
  if( readState == 42 )
  {
    readState = 0;
    switch( atoi(tabPar) )
    {
      case CARD_SETPRINTFILE : setCardPrintFile( atoi(tabWart) );
        break;
    }
  }
}


